BaseLocalAnomalyScore — skchange

Skip to main content
`Ctrl`+`K`

# BaseLocalAnomalyScore

class BaseLocalAnomalyScore

  Base class template for local anomaly scores.

  Local anomaly scores are used to detect anomalies in a time series or sequence by evaluating the deviation of the data distribution within a subinterval of a larger, local interval.

  Attributes:
    `is_fitted`

      Whether fit has been called.

    `min_size`

      Minimum valid size of an interval to evaluate.

  Methods

  `check_is_fitted`() | Check if the estimator has been fitted.
  `clone`() | Obtain a clone of the object with same hyper-parameters.
  `clone_tags`(estimator[, tag_names]) | Clone tags from another estimator as dynamic override.
  `create_test_instance`([parameter_set]) | Construct Estimator instance if possible.
  `create_test_instances_and_names`([parameter_set]) | Create list of all test instances and a list of names for them.
  `evaluate`(cuts) | Evaluate the score according to a set of cuts.
  `fit`(X[, y]) | Fit the interval scorer to the training data.
  `get_class_tag`(tag_name[, tag_value_default]) | Get a class tag's value.
  `get_class_tags`() | Get class tags from the class and all its parent classes.
  `get_config`() | Get config flags for self.
  `get_fitted_params`([deep]) | Get fitted parameters.
  `get_param_defaults`() | Get object's parameter defaults.
  `get_param_names`() | Get object's parameter names.
  `get_params`([deep]) | Get a dict of parameters values for this object.
  `get_tag`(tag_name[, tag_value_default, ...]) | Get tag value from estimator class and dynamic tag overrides.
  `get_tags`() | Get tags from estimator class and dynamic tag overrides.
  `get_test_params`([parameter_set]) | Return testing parameter settings for the estimator.
  `is_composite`() | Check if the object is composed of other BaseObjects.
  `load_from_path`(serial) | Load object from file location.
  `load_from_serial`(serial) | Load object from serialized memory container.
  `reset`() | Reset the object to a clean post-init state.
  `save`([path, serialization_format]) | Save serialized self to bytes-like object or to (.zip) file.
  `set_config`(**config_dict) | Set config flags to given values.
  `set_params`(**params) | Set the parameters of this object.
  `set_tags`(**tag_dict) | Set dynamic tags to given values.

  check_is_fitted()

    Check if the estimator has been fitted.

    Inspects object’s _is_fitted attribute that should initialize to False during object construction, and be set to True in calls to an object’s fit method.

    Raises:
      NotFittedError

        If the estimator has not been fitted yet.

  clone()

    Obtain a clone of the object with same hyper-parameters.

    A clone is a different object without shared references, in post-init state. This function is equivalent to returning sklearn.clone of self.

    Raises:
      RuntimeError if the clone is non-conforming, due to faulty `__init__`.

    Notes

    If successful, equal in value to `type(self)(**self.get_params(deep=False))`.

  clone_tags(estimator, tag_names=None)

    Clone tags from another estimator as dynamic override.

    Parameters:
      estimatorestimator inheriting from :class:BaseEstimator
      tag_namesstr or list of str, default = None

        Names of tags to clone. If None then all tags in estimator are used as tag_names.

    Returns:
      Self

        Reference to self.

    Notes

    Changes object state by setting tag values in tag_set from estimator as dynamic tags in self.

  classmethod create_test_instance(parameter_set='default')

    Construct Estimator instance if possible.

    Parameters:
      parameter_setstr, default=”default”

        Name of the set of test parameters to return, for use in tests. If no special parameters are defined for a value, will return “default” set.

    Returns:
      instanceinstance of the class with default parameters

    Notes

    get_test_params can return dict or list of dict. This function takes first or single dict that get_test_params returns, and constructs the object with that.

  classmethod create_test_instances_and_names(parameter_set='default')

    Create list of all test instances and a list of names for them.

    Parameters:
      parameter_setstr, default=”default”

        Name of the set of test parameters to return, for use in tests. If no special parameters are defined for a value, will return “default” set.

    Returns:
      objslist of instances of cls

        i-th instance is cls(**cls.get_test_params()[i])

      nameslist of str, same length as objs

        i-th element is name of i-th instance of obj in tests convention is {cls.__name__}-{i} if more than one instance otherwise {cls.__name__}

      parameter_setstr, default=”default”

        Name of the set of test parameters to return, for use in tests. If no special parameters are defined for a value, will return “default” set.

  evaluate(cuts: _SupportsArray[dtype[Any]] | _NestedSequence[_SupportsArray[dtype[Any]]] | bool | int | float | complex | str | bytes | _NestedSequence[bool | int | float | complex | str | bytes]) → ndarray

    Evaluate the score according to a set of cuts.

    Parameters:
      cutsArrayLike

        A 2D array of integer location-based cuts to evaluate where each row gives a single cut specification. If a 1D array is passed, it is assumed to be a row vector. Each cut divide the data into one or more intervals for evaluation and may contain multiple entries representing for example the start and end of the interval, and potentially split points within the interval. Each cut must be sorted in increasing order. Subclasses specify the further expected structure of the cuts array and how it is used internally to evaluate the score.

    Returns:
      scoresnp.ndarray

        A 2D array of scores. One row for each row in cuts.

  fit(X, y=None)

    Fit the interval scorer to the training data.

    Parameters:
      Xpd.Series, pd.DataFrame, or np.ndarray

        Data to score.

      yNone

        Ignored. Included for API consistency by convention.

    Returns:
      self

        Reference to self.

    Notes

    Updates the fitted model and sets attributes ending in “_”.

  classmethod get_class_tag(tag_name, tag_value_default=None)

    Get a class tag’s value.

    Does not return information from dynamic tags (set via set_tags or clone_tags) that are defined on instances.

    Parameters:
      tag_namestr

        Name of tag value.

      tag_value_defaultany

        Default/fallback value if tag is not found.

    Returns:
      tag_value

        Value of the tag_name tag in self. If not found, returns tag_value_default.

  classmethod get_class_tags()

    Get class tags from the class and all its parent classes.

    Retrieves tag: value pairs from _tags class attribute. Does not return information from dynamic tags (set via set_tags or clone_tags) that are defined on instances.

    Returns:
      collected_tagsdict

        Dictionary of class tag name: tag value pairs. Collected from _tags class attribute via nested inheritance.

  get_config()

    Get config flags for self.

    Returns:
      config_dictdict

        Dictionary of config name : config value pairs. Collected from _config class attribute via nested inheritance and then any overrides and new tags from _onfig_dynamic object attribute.

  get_fitted_params(deep=True)

    Get fitted parameters.

    State required:

      Requires state to be “fitted”.

    Parameters:
      deepbool, default=True

        Whether to return fitted parameters of components.

        - If True, will return a dict of parameter name : value for this object, including fitted parameters of fittable components (= BaseEstimator-valued parameters).

        - If False, will return a dict of parameter name : value for this object, but not include fitted parameters of components.

    Returns:
      fitted_paramsdict with str-valued keys

        Dictionary of fitted parameters, paramname : paramvalue keys-value pairs include:

        - always: all fitted parameters of this object, as via get_param_names values are fitted parameter value for that key, of this object

        - if deep=True, also contains keys/value pairs of component parameters parameters of components are indexed as [componentname]__[paramname] all parameters of componentname appear as paramname with its value

        - if deep=True, also contains arbitrary levels of component recursion, e.g., [componentname]__[componentcomponentname]__[paramname], etc

  classmethod get_param_defaults()

    Get object’s parameter defaults.

    Returns:
      default_dict: dict[str, Any]

        Keys are all parameters of cls that have a default defined in __init__ values are the defaults, as defined in __init__.

  classmethod get_param_names()

    Get object’s parameter names.

    Returns:
      param_names: list[str]

        Alphabetically sorted list of parameter names of cls.

  get_params(deep=True)

    Get a dict of parameters values for this object.

    Parameters:
      deepbool, default=True

        Whether to return parameters of components.

        - If True, will return a dict of parameter name : value for this object, including parameters of components (= BaseObject-valued parameters).

        - If False, will return a dict of parameter name : value for this object, but not include parameters of components.

    Returns:
      paramsdict with str-valued keys

        Dictionary of parameters, paramname : paramvalue keys-value pairs include:

        - always: all parameters of this object, as via get_param_names values are parameter value for that key, of this object values are always identical to values passed at construction

        - if deep=True, also contains keys/value pairs of component parameters parameters of components are indexed as [componentname]__[paramname] all parameters of componentname appear as paramname with its value

        - if deep=True, also contains arbitrary levels of component recursion, e.g., [componentname]__[componentcomponentname]__[paramname], etc

  get_tag(tag_name, tag_value_default=None, raise_error=True)

    Get tag value from estimator class and dynamic tag overrides.

    Parameters:
      tag_namestr

        Name of tag to be retrieved

      tag_value_defaultany type, optional; default=None

        Default/fallback value if tag is not found

      raise_errorbool

        whether a ValueError is raised when the tag is not found

    Returns:
      tag_valueAny

        Value of the tag_name tag in self. If not found, returns an error if raise_error is True, otherwise it returns tag_value_default.

    Raises:
      ValueError if raise_error is True i.e. if tag_name is not in
      self.get_tags().keys()
  get_tags()

    Get tags from estimator class and dynamic tag overrides.

    Returns:
      collected_tagsdict

        Dictionary of tag name : tag value pairs. Collected from _tags class attribute via nested inheritance and then any overrides and new tags from _tags_dynamic object attribute.

  classmethod get_test_params(parameter_set='default')

    Return testing parameter settings for the estimator.

    Parameters:
      parameter_setstr, default=”default”

        Name of the set of test parameters to return, for use in tests. If no special parameters are defined for a value, will return “default” set.

    Returns:
      paramsdict or list of dict, default = {}

        Parameters to create testing instances of the class Each dict are parameters to construct an “interesting” test instance, i.e., MyClass(**params) or MyClass(**params[i]) creates a valid test instance. create_test_instance uses the first (or only) dictionary in params

  is_composite()

    Check if the object is composed of other BaseObjects.

    A composite object is an object which contains objects, as parameters. Called on an instance, since this may differ by instance.

    Returns:
      composite: bool

        Whether an object has any parameters whose values are BaseObjects.

  property is_fitted

    Whether fit has been called.

    Inspects object’s _is_fitted attribute that should initialize to False during object construction, and be set to True in calls to an object’s fit method.

    Returns:
      bool

        Whether the estimator has been fit.

  classmethod load_from_path(serial)

    Load object from file location.

    Parameters:
      serialresult of ZipFile(path).open(“object)
    Returns:
      deserialized self resulting in output at `path`, of `cls.save(path)`
  classmethod load_from_serial(serial)

    Load object from serialized memory container.

    Parameters:
      serial1st element of output of `cls.save(None)`
    Returns:
      deserialized self resulting in output `serial`, of `cls.save(None)`
  property min_size: int | None

    Minimum valid size of an interval to evaluate.

    The size of each interval is by default defined as np.diff(cuts[i, ]). Subclasses can override the min_size to mean something else, for example in cases where intervals are combined before evaluation or the cuts specify disjoint intervals.

    Returns:
      int or None

        The minimum valid size of an interval to evaluate. If None, it is unknown what the minimum size is. E.g., the scorer may need to be fitted first to determine the minimum size.

  reset()

    Reset the object to a clean post-init state.

    Using reset, runs __init__ with current values of hyper-parameters (result of get_params). This Removes any object attributes, except:

    > - hyper-parameters = arguments of __init__

    > - object attributes containing double-underscores, i.e., the string “__”

    Class and object methods, and class attributes are also unaffected.

    Returns:
      self

        Instance of class reset to a clean post-init state but retaining the current hyper-parameter values.

    Notes

    Equivalent to sklearn.clone but overwrites self. After self.reset() call, self is equal in value to type(self)(**self.get_params(deep=False))

  save(path=None, serialization_format='pickle')

    Save serialized self to bytes-like object or to (.zip) file.

    Behaviour: if `path` is None, returns an in-memory serialized self if `path` is a file location, stores self at that location as a zip file

    saved files are zip files with following contents: _metadata - contains class of self, i.e., type(self) _obj - serialized self. This class uses the default serialization (pickle).

    Parameters:
      pathNone or file location (str or Path)

        if None, self is saved to an in-memory object if file location, self is saved to that file location. If:

        > path=”estimator” then a zip file `estimator.zip` will be made at cwd. path=”/home/stored/estimator” then a zip file `estimator.zip` will be stored in `/home/stored/`.

      serialization_format: str, default = “pickle”

        Module to use for serialization. The available options are “pickle” and “cloudpickle”. Note that non-default formats might require installation of other soft dependencies.

    Returns:
      if `path` is None - in-memory serialized self
      if `path` is file location - ZipFile with reference to the file
  set_config(**config_dict)

    Set config flags to given values.

    Parameters:
      config_dictdict

        Dictionary of config name : config value pairs. Valid configs, values, and their meaning is listed below:

      displaystr, “diagram” (default), or “text”

        how jupyter kernels display instances of self

        - “diagram” = html box diagram representation

        - “text” = string printout

      print_changed_onlybool, default=True

        whether printing of self lists only self-parameters that differ from defaults (False), or all parameter names and values (False). Does not nest, i.e., only affects self and not component estimators.

      warningsstr, “on” (default), or “off”

        whether to raise warnings, affects warnings from sktime only

        - “on” = will raise warnings from sktime

        - “off” = will not raise warnings from sktime

      backend:parallelstr, optional, default=”None”

        backend to use for parallelization when broadcasting/vectorizing, one of

        - “None”: executes loop sequentally, simple list comprehension

        - “loky”, “multiprocessing” and “threading”: uses `joblib.Parallel`

        - “joblib”: custom and 3rd party `joblib` backends, e.g., `spark`

        - “dask”: uses `dask`, requires `dask` package in environment

      backend:parallel:paramsdict, optional, default={} (no parameters passed)

        additional parameters passed to the parallelization backend as config. Valid keys depend on the value of `backend:parallel`:

        - “None”: no additional parameters, `backend_params` is ignored

        - “loky”, “multiprocessing” and “threading”: default `joblib` backends any valid keys for `joblib.Parallel` can be passed here, e.g., `n_jobs`, with the exception of `backend` which is directly controlled by `backend`. If `n_jobs` is not passed, it will default to `-1`, other parameters will default to `joblib` defaults.

        - “joblib”: custom and 3rd party `joblib` backends, e.g., `spark`. Any valid keys for `joblib.Parallel` can be passed here, e.g., `n_jobs`, `backend` must be passed as a key of `backend_params` in this case. If `n_jobs` is not passed, it will default to `-1`, other parameters will default to `joblib` defaults.

        - “dask”: any valid keys for `dask.compute` can be passed, e.g., `scheduler`

    Returns:
      selfreference to self.

    Notes

    Changes object state, copies configs in config_dict to self._config_dynamic.

  set_params(**params)

    Set the parameters of this object.

    The method works on simple estimators as well as on composite objects. Parameter key strings `<component>__<parameter>` can be used for composites, i.e., objects that contain other objects, to access `<parameter>` in the component `<component>`. The string `<parameter>`, without `<component>__`, can also be used if this makes the reference unambiguous, e.g., there are no two parameters of components with the name `<parameter>`.

    Parameters:
      **paramsdict

        BaseObject parameters, keys must be `<component>__<parameter>` strings. __ suffixes can alias full strings, if unique among get_params keys.

    Returns:
      selfreference to self (after parameters have been set)
  set_tags(**tag_dict)

    Set dynamic tags to given values.

    Parameters:
      **tag_dictdict

        Dictionary of tag name: tag value pairs.

    Returns:
      Self

        Reference to self.

    Notes

    Changes object state by settting tag values in tag_dict as dynamic tags in self.